'use strict';

var KindaObject = require('kinda-object');
var KindaLog = require('kinda-log');
var util = require('kinda-util').create();

var KindaNotifier = KindaObject.extend('KindaNotifier', function () {
  // options:
  //   sender
  //   appName
  //   hostName
  //   targets
  //   includeEnvironment (default: true)
  //   log
  this.creator = function () {
    var options = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

    var sender = options.sender;
    if (!sender) {
      var includeEnvironment = options.includeEnvironment;
      if (includeEnvironment == null) includeEnvironment = true;

      var appName = options.appName;
      if (includeEnvironment) {
        if (appName) appName += '.';else appName = '';
        appName += util.getEnvironment();
      }

      var hostName = options.hostName;
      if (!hostName) hostName = util.getHostName();

      sender = appName;
      if (sender) sender += '@';else sender = '';
      sender += hostName;
    }
    this.sender = sender;

    var targets = options.targets;
    if (!targets) {
      var target = KindaNotifier.NodeNotifierTarget.create();
      targets = [target];
    }
    this.targets = targets;

    var log = options.log;
    if (!KindaLog.isClassOf(log)) log = KindaLog.create(log);
    this.log = log;
  };

  this.addTarget = function (target) {
    this.targets.push(target);
  };

  this.send = function (title, message) {
    var _this = this;

    (function callee$2$0() {
      return regeneratorRuntime.async(function callee$2$0$(context$3$0) {
        while (1) switch (context$3$0.prev = context$3$0.next) {
          case 0:
            context$3$0.next = 2;
            return regeneratorRuntime.awrap(this.sendAndWaitUntilCompleted(title, message));

          case 2:
          case 'end':
            return context$3$0.stop();
        }
      }, null, this);
    }).call(this)['catch'](function (err) {
      _this.log.error(err.stack || err);
    });
  };

  this.sendAndWaitUntilCompleted = function callee$1$0(title, message) {
    var _iteratorNormalCompletion, _didIteratorError, _iteratorError, _iterator, _step, target;

    return regeneratorRuntime.async(function callee$1$0$(context$2$0) {
      while (1) switch (context$2$0.prev = context$2$0.next) {
        case 0:
          if (!message) {
            message = title;
            title = undefined;
          }

          if (message) {
            context$2$0.next = 3;
            break;
          }

          throw new Error('a \'message\' is required');

        case 3:
          _iteratorNormalCompletion = true;
          _didIteratorError = false;
          _iteratorError = undefined;
          context$2$0.prev = 6;
          _iterator = this.targets[Symbol.iterator]();

        case 8:
          if (_iteratorNormalCompletion = (_step = _iterator.next()).done) {
            context$2$0.next = 15;
            break;
          }

          target = _step.value;
          context$2$0.next = 12;
          return regeneratorRuntime.awrap(target.send(this.sender, title, message));

        case 12:
          _iteratorNormalCompletion = true;
          context$2$0.next = 8;
          break;

        case 15:
          context$2$0.next = 21;
          break;

        case 17:
          context$2$0.prev = 17;
          context$2$0.t0 = context$2$0['catch'](6);
          _didIteratorError = true;
          _iteratorError = context$2$0.t0;

        case 21:
          context$2$0.prev = 21;
          context$2$0.prev = 22;

          if (!_iteratorNormalCompletion && _iterator['return']) {
            _iterator['return']();
          }

        case 24:
          context$2$0.prev = 24;

          if (!_didIteratorError) {
            context$2$0.next = 27;
            break;
          }

          throw _iteratorError;

        case 27:
          return context$2$0.finish(24);

        case 28:
          return context$2$0.finish(21);

        case 29:
        case 'end':
          return context$2$0.stop();
      }
    }, null, this, [[6, 17, 21, 29], [22,, 24, 28]]);
  };
});

KindaNotifier.SlackIncomingWebhookTarget = require('./targets/slack-incoming-webhook');
KindaNotifier.NodeNotifierTarget = require('./targets/node-notifier');

module.exports = KindaNotifier;